import configparser

from Redis import Redis


class UrlShortener:
    def __init__(self):
        self.redis = Redis()
        if not self.redis.does_key_exist('id'):
            self.redis.add_key_value('id', '1')
        config = configparser.ConfigParser()
        config.read('config.ini')
        self.short_url = config['config']['short_url']
        self.accepted_char = config['config']['accepted_char']

    def shorten_url(self, original_url):
        if (self.redis.does_key_exist(original_url)):
            url_id = int(self.redis.get_value_by_key(original_url))
            shorten_url = self.url_encoder(url_id)
        else:
            url_id = int(self.redis.get_value_by_key('id'))
            self.redis.add_key_value(original_url, url_id)
            shorten_url = self.url_encoder(url_id)
            url_id += 1
            self.redis.add_key_value('id', str(url_id))
        return self.short_url+shorten_url

    def url_encoder(self, id):
        characters = self.accepted_char
        base = len(characters)
        encoden_url = []
        while id > 0:
            val = id % base
            encoden_url.append(characters[val])
            id = (id // base)
        return "".join(encoden_url[::-1])



import configparser
from io import BytesIO
import json

from http.server import HTTPServer, BaseHTTPRequestHandler

from UrlShortener import UrlShortener


class HttpHandler(BaseHTTPRequestHandler):
    def __init__(self):
        config = configparser.ConfigParser()
        config.read('handler.ini')
        self.linsten_ip = config['httphandler']['ip']
        self.listen_port = config['httphandler']['port']

    def do_GET(self):
        self.send_response(400)
        self.end_headers()
        self.wfile.write(b'Bad request method!')

    def do_POST(self):
        content_length = int(self.headers['Content-Length'])
        body = self.rfile.read(content_length)
        response = BytesIO()
        request_body_dict = json.loads(body.decode('utf8'))
        if('url' in request_body_dict):
            request_url = request_body_dict['url']
            url_shortener = UrlShortener()
            shortend_url = url_shortener.shorten_url(str(request_url))
            json_response = json.dumps({request_url: shortend_url})
            self.send_response(200)
            self.end_headers()
            response.write(json_response.encode('utf8'))
            self.wfile.write(response.getvalue())
        else:
            self.send_response(400)
            self.end_headers()
            response.write(b'Bad request param!')
            self.wfile.write(response.getvalue())

    def handle_requests(self):
        http_handler = HTTPServer((self.listen_port, self.listen_port),
                                  HttpHandler)
        http_handler.serve_forever()



import unittest
import mock
from UrlShortener import UrlShortener
from Redis import Redis
import re


class TestUrlShortner(unittest.TestCase):
    url_shortner = UrlShortener()

    def test_url_encoder_function(self):
        """
        Test url_encoder function behavior
        """
        encoded_url = self.url_shortner.url_encoder(10000000)
        self.assertEqual(encoded_url, 'oCba')

    @mock.patch.object(Redis, 'does_key_exist', return_value=True)
    @mock.patch.object(Redis, 'get_value_by_key', return_value=237)
    def test_shorten_url_key_exist(self, *_):
        """
        Test shorten_url when url already exist
        """
        url_key_id = self.url_shortner.shorten_url('test')
        regex_search = bool(re.search(r'.*/3f$', url_key_id))
        self.assertEqual(regex_search, True)

    @mock.patch.object(Redis, 'does_key_exist', return_value=False)
    @mock.patch.object(Redis, 'get_value_by_key')
    @mock.patch.object(Redis, 'add_key_value')
    @mock.patch.object(UrlShortener, 'url_encoder', return_value='aB3')
    def test_shorten_url_add_key(self, *_):
        """
        Test shorten_url when url doesn't exist
        """
        shotrened_url = self.url_shortner.shorten_url('test')
        regex_search = bool(re.search(r'.*/aB3$', shotrened_url))
        self.assertEqual(regex_search, True)


if __name__ == '__main__':
    unittest.main()



import redis
import configparser


class Redis:
    def __init__(self):
        self.redis_db = self.connect_to_redis()
        config = configparser.ConfigParser()
        config.read('db.ini')
        self.redis_host = config['redis']['host']
        self.redis_port = config['redis']['port']
        self.redis_pass = config['redis']['pass']

    def connect_to_redis(self):
        return redis.Redis(
                            host=self.redis_host,
                            port=self.redis_port,
                            password=None)

    def add_key_value(self, key, value):
        self.redis_db.set(key, value)

    def get_value_by_key(self, key):
        return self.redis_db.get(key)

    def does_key_exist(self, key):
        print(key)
        if(self.redis_db.exists(key)):
            return True
        return False

    def remove_key(self, key):
        self.redis_db.delete(key)



import pathlib
from setuptools import setup

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name="urlshortener",
    version="1.0.0",
    description="shorten long urls",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/3ep-one/urlshortener",
    author="Sepehr Rafiei",
    author_email="sepehr.rafiei.k@ut.ac.ir",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["urlshortener"],
    include_package_data=True,
    install_requires=["redis", "HTTPServer"],
    entry_points={
        "console_scripts": [
            "urlshortener=urlshortener.__main__:main",
        ]
    },
)
